@@ -31,6 +31,7 @@
 #include <linux/version.h>
 #include <linux/locks.h>
 #include <linux/list.h>
+#include <linux/seq_file.h>
 #include "cifsfs.h"
 #include "cifspdu.h"
 #define DECLARE_GLOBALS_HERE
@@ -49,6 +50,8 @@ int traceSMB = 0;
 unsigned int oplockEnabled = 0;
 unsigned int multiuser_mount = 0;
 unsigned int extended_security = 0;
+unsigned int ntlmv2_support = 0;
+unsigned int sign_CIFS_PDUs = 0;
 
 extern int cifs_mount(struct super_block *, struct cifs_sb_info *, char *);
 extern int cifs_umount(struct super_block *, struct cifs_sb_info *);
@@ -62,8 +65,7 @@ struct super_block *cifs_read_super(struct super_block *sb,
 	struct cifs_sb_info *cifs_sb;
 	int rc = 0;
 
-	/* dump_mem(" cifs_fs_type ", &cifs_fs_type, sizeof(cifs_fs_type)); */
-	cifs_sb = (struct cifs_sb_info *) &sb->u;
+	cifs_sb = CIFS_SB(sb);
 	cifs_sb->local_nls = load_nls_default();	/* needed for ASCII cp to Unicode converts */
 	rc = cifs_mount(sb, cifs_sb, data);
 
@@ -91,17 +93,16 @@ struct super_block *cifs_read_super(struct super_block *sb,
 
 	return sb;
 
-      out_no_root:
+out_no_root:
 	cEVENT(1, ("cifs_read_super: get root inode failed\n"));
 	if (inode)
 		iput(inode);
 
 /*	rc = cifs_umount(sb);  BB is CIFS unmount routine needed? */
 	if (rc) {
-		cERROR(1,
-		       ("cifs_umount failed with return code %d\n", rc));
+		cERROR(1, ("cifs_mount failed with no root inode"));
 	}
-      out_mount_failed:
+out_mount_failed:
 	sb->s_dev = 0;
 	return 0;
 }
@@ -112,8 +113,12 @@ void cifs_put_super(struct super_block *sb)
 	struct cifs_sb_info *cifs_sb;
 
 	cFYI(1, ("In cifs_put_super\n"));
-	cifs_sb = (struct cifs_sb_info *) &sb->u;
-	rc = cifs_umount(sb, cifs_sb);	/* BB add in missing function Make sure to free waiting procs by preventing new sends on session if this tcon is the last one and also waking those blocked on server->response_q before exiting.  Also remember to only kill the SMB session (ses struct) by sending uloggoffX if this is the last tconn on the SMB session and kill the socket (tcp session) if this is the last SMB session on the socket BB (tcp_server_info struct */
+	cifs_sb = CIFS_SB(sb);
+	if(cifs_sb == NULL) {
+		cFYI(1,("\nEmpty cifs superblock info passed to unmount"));
+		return;
+	}
+	rc = cifs_umount(sb, cifs_sb); 
 	if (rc) {
 		cERROR(1,
 		       ("cifs_umount failed with return code %d\n", rc));
@@ -131,16 +136,16 @@ int cifs_statfs(struct super_block *sb, struct statfs *buf)
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) &(sb->u);
+	cifs_sb = CIFS_SB(sb);
 	pTcon = cifs_sb->tcon;
 
 	buf->f_type = CIFS_MAGIC_NUMBER;
 
 	/* instead could get the real value via SMB_QUERY_FS_ATTRIBUTE_INFO */
-	buf->f_namelen = PATH_MAX;	/* PATH_MAX may be too long - it would presumably be the
-					   length of total path, note that some servers may be 
+	buf->f_namelen = PATH_MAX;	/* PATH_MAX may be too long - it would presumably
+					   be length of total path, note that some servers may be 
 					   able to support more than this, but best to be safe
-					   since Win2k among others will not work with long filenames */
+					   since Win2k and others can not handle very long filenames */
 	buf->f_files = 0;	/* undefined */
 	buf->f_ffree = -1;	/* unlimited */
 
@@ -149,27 +154,17 @@ int cifs_statfs(struct super_block *sb, struct statfs *buf)
 	/*     
 	   int f_type;
 	   __fsid_t f_fsid;
-   int f_namelen;  *//* BB get this/these from info put in tcon struct at mount time with call to QFSAttrInfo */
+	   int f_namelen;  */
+	/* BB get from info put in tcon struct at mount time with call to QFSAttrInfo */
 
+	FreeXid(xid);
 	return 0;		/* always return success? what if volume is no longer available? */
 }
 
-struct address_space_operations cifs_addr_ops = {
-	/* none needed yet */
-
-};
-
 struct super_operations cifs_super_ops = {
-
-/*clear_inode:  cifs_clear_inode,
-  write_inode:  cifs_write_inode,*//* probably unneeded in our case */
-
 	read_inode:cifs_read_inode,
-/* put_inode:    cifs_put_inode, *//* BB do not need to know about decrement of usage count ? */
-/* delete_inode: cifs_delete_inode,  *//* BB add back in delete_inode code when safe distributed inode caching - ie FindNotify added */
 	put_super:cifs_put_super,
 	statfs:cifs_statfs,
-
 	/*  umount_begin: cifs_umount_begin,
 	   dirty_inode:  cifs_dirty_inode,
    remount_fs:   cifs_remount_fs, *//* TBD */
@@ -205,12 +200,13 @@ struct inode_operations cifs_symlink_inode_ops = {
 };
 
 struct file_operations cifs_file_ops = {
-	read:cifs_read,
-	write:cifs_write,
+	read:generic_file_read,
+	write:generic_file_write,
 	open:cifs_open,
 	release:cifs_close,
 	lock:cifs_lock,
 	fsync:cifs_fsync,
+	mmap:cifs_file_mmap,
 };
 
 struct file_operations cifs_dir_ops = {
@@ -219,8 +215,53 @@ struct file_operations cifs_dir_ops = {
 	release:cifs_closedir,
 };
 
+kmem_cache_t *cifs_req_cachep;
+kmem_cache_t *cifs_mid_cachep;
+
+int
+cifs_init_request_bufs(void)
+{
+	cifs_req_cachep = kmem_cache_create("cifs_request",
+					    CIFS_MAX_MSGSIZE +
+					    MAX_CIFS_HDR_SIZE, 0,
+					    SLAB_HWCACHE_ALIGN, NULL, NULL);
+	if (cifs_req_cachep == NULL)
+		return -ENOMEM;
+
+	return 0;
+}
+
+void
+cifs_destroy_request_bufs(void)
+{
+	if (kmem_cache_destroy(cifs_req_cachep))
+		printk(KERN_INFO
+		       "cifs_destroy_request_cache: error not all structures were freed\n");
+}
+
+int
+cifs_init_mids(void)
+{
+	cifs_mid_cachep = kmem_cache_create("cifs_mpx_ids",
+					    sizeof (struct mid_q_entry), 0,
+					    SLAB_HWCACHE_ALIGN, NULL, NULL);
+	if (cifs_mid_cachep == NULL)
+		return -ENOMEM;
+
+	return 0;
+}
+
+void
+cifs_destroy_mids(void)
+{
+	if (kmem_cache_destroy(cifs_mid_cachep))
+		printk(KERN_INFO
+		       "cifs_destroy_mids: error not all structures were freed\n");
+}
+
 static int __init init_cifs(void)
 {
+	int rc = 0;
 #if CONFIG_PROC_FS
 	cifs_proc_init();
 #endif
@@ -235,10 +276,25 @@ static int __init init_cifs(void)
 	atomic_set(&tconInfoAllocCount, 0);
 	atomic_set(&bufAllocCount, 0);
 	atomic_set(&midCount, 0);
+	GlobalCurrentXid = 0;
 	GlobalTotalActiveXid = 0;
 	GlobalMaxActiveXid = 0;
+	GlobalSMBSeslock = RW_LOCK_UNLOCKED;
+	GlobalMid_Lock = RW_LOCK_UNLOCKED;
 
-	return register_filesystem(&cifs_fs_type);
+	rc = cifs_init_mids();
+	if (!rc) {
+		rc = cifs_init_request_bufs();
+		if (!rc) {
+			rc = register_filesystem(&cifs_fs_type);
+			if (!rc)
+				return rc;	/* Success */
+			else
+				cifs_destroy_request_bufs();
+		}
+		cifs_destroy_mids();
+	}
+	return rc;
 }
 
 static void __exit exit_cifs(void)
@@ -247,8 +303,9 @@ static void __exit exit_cifs(void)
 #if CONFIG_PROC_FS
 	cifs_proc_clean();
 #endif
-/* BB check to see if there is additional memory freeing we need to do here e.g. freeing from list_heads etc. BB */
 	unregister_filesystem(&cifs_fs_type);
+	cifs_destroy_mids();
+	cifs_destroy_request_bufs();
 }
 
 MODULE_AUTHOR("Steve French <sfrench@us.ibm.com>");
@@ -256,6 +313,6 @@ MODULE_AUTHOR("Steve French <sfrench@us.ibm.com>");
 MODULE_LICENSE("GPL");		/* combination of LGPL + GPL source behaves as GPL */
 #endif
 MODULE_DESCRIPTION
-    ("VFS support for Common Internet File System servers such as Windows, Samba and those that comply with the SNIA CIFS Specification");
+    ("VFS to access servers complying with the SNIA CIFS Specification e.g. Samba and Windows");
 module_init(init_cifs)
-    module_exit(exit_cifs)
+module_exit(exit_cifs)
