@@ -30,7 +30,8 @@
 #include "cifs_debug.h"
 #include "cifs_fs_sb.h"
 
-void renew_parental_timestamps(struct dentry *direntry)
+void
+renew_parental_timestamps(struct dentry *direntry)
 {
 	/* BB check if there is a way to get the kernel to do this or if we really need this */
 	do {
@@ -40,7 +41,8 @@ void renew_parental_timestamps(struct dentry *direntry)
 }
 
 /* Note: caller must free return buffer */
-char *build_path_from_dentry(struct dentry *direntry)
+char *
+build_path_from_dentry(struct dentry *direntry)
 {
 	struct dentry *temp;
 	int namelen = 0;
@@ -76,7 +78,8 @@ char *build_path_from_dentry(struct dentry *direntry)
 	return full_path;
 }
 
-char *build_wildcard_path_from_dentry(struct dentry *direntry)
+char *
+build_wildcard_path_from_dentry(struct dentry *direntry)
 {
 	struct dentry *temp;
 	int namelen = 0;
@@ -116,7 +119,8 @@ char *build_wildcard_path_from_dentry(struct dentry *direntry)
 
 /* Inode operations in similar order to how they appear in the Linux file fs.h */
 
-int cifs_create(struct inode *inode, struct dentry *direntry, int mode)
+int
+cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 {
 	int rc = -ENOENT;
 	int xid;
@@ -129,15 +133,14 @@ int cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_path_from_dentry(direntry);
 
 	/* BB add processing for setting the equivalent of mode - e.g. via CreateX with ACLs */
 
-	rc = CIFSSMBOpen(xid, pTcon, full_path, FILE_OVERWRITE_IF,
-			 GENERIC_ALL
+	rc = CIFSSMBOpen(xid, pTcon, full_path, FILE_OVERWRITE_IF, GENERIC_ALL
 			 /* 0x20197 was used previously */ , mode,
 			 &fileHandle, &oplock, cifs_sb->local_nls);
 	if (rc) {
@@ -156,10 +159,14 @@ int cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 			      rc));
 			/* close handle */
 		} else {
-			direntry->d_op = &cifs_dentry_ops;	
+			direntry->d_op = &cifs_dentry_ops;
 			d_instantiate(direntry, newinode);
 		}
-		CIFSSMBClose(xid, pTcon, fileHandle);	/* BB chain the close with the NTCreateX to narrow window */
+		CIFSSMBClose(xid, pTcon, fileHandle);	
+        /* BB In the future chain close with the NTCreateX to narrow window */
+        
+        if(newinode)
+            newinode->i_mode = mode;
 	}
 	if (full_path)
 		kfree(full_path);
@@ -168,8 +175,8 @@ int cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 	return rc;
 }
 
-struct dentry *cifs_lookup(struct inode *parent_dir_inode,
-			   struct dentry *direntry)
+struct dentry *
+cifs_lookup(struct inode *parent_dir_inode, struct dentry *direntry)
 {
 	int rc, xid;
 	struct cifs_sb_info *cifs_sb;
@@ -183,13 +190,11 @@ struct dentry *cifs_lookup(struct inode *parent_dir_inode,
 	     (" parent inode = 0x%p name is: %s and dentry = 0x%p",
 	      parent_dir_inode, direntry->d_name.name, direntry));
 
-
 	/* BB Add check of incoming data - e.g. frame not longer than maximum SMB - let server check the namelen BB */
 
-
 	/* check whether path exists */
 
-	cifs_sb = (struct cifs_sb_info *) (&(parent_dir_inode->i_sb->u));
+	cifs_sb = CIFS_SB(parent_dir_inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_path_from_dentry(direntry);
@@ -199,8 +204,7 @@ struct dentry *cifs_lookup(struct inode *parent_dir_inode,
 		cFYI(1, (" NULL inode in lookup"));
 	}
 	cFYI(1,
-	     (" Full path: %s inode = 0x%p\n", full_path,
-	      direntry->d_inode));
+	     (" Full path: %s inode = 0x%p\n", full_path, direntry->d_inode));
 	if (pTcon->ses->capabilities & CAP_UNIX)
 		rc = cifs_get_inode_info_unix(&newInode, full_path,
 					      parent_dir_inode->i_sb);
@@ -231,7 +235,8 @@ struct dentry *cifs_lookup(struct inode *parent_dir_inode,
 	return ERR_PTR(rc);
 }
 
-int cifs_dir_open(struct inode *inode, struct file *file)
+int
+cifs_dir_open(struct inode *inode, struct file *file)
 {				/* NB: currently unused since searches are opened in readdir */
 	int rc = 0;
 	int xid;
@@ -239,10 +244,9 @@ int cifs_dir_open(struct inode *inode, struct file *file)
 	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
 
-
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_wildcard_path_from_dentry(file->f_dentry);
@@ -255,23 +259,31 @@ int cifs_dir_open(struct inode *inode, struct file *file)
 	return rc;
 }
 
-
-static int cifs_d_revalidate(struct dentry *direntry, int flags)
+static int
+cifs_d_revalidate(struct dentry *direntry, int flags)
 {
 	int isValid = 1;
 
+/*	lock_kernel(); *//* surely we do not want to lock the kernel for a whole network round trip which could take seconds */
+
 	if (direntry->d_inode) {
 		cFYI(1,
 		     ("In cifs_d_revalidate, name = %s and inode = 0x%p with count %d with time %ld and dentry 0x%p with time %ld\n",
 		      direntry->d_name.name, direntry->d_inode,
 		      direntry->d_inode->i_count.counter,
-		      direntry->d_inode->i_atime, direntry,
-		      direntry->d_time));
+		      direntry->d_inode->i_atime, direntry, direntry->d_time));
+		if (cifs_revalidate(direntry)) {
+			/* unlock_kernel(); */
+			return 0;
+		}
 	} else {
 		cFYI(1,
 		     ("In cifs_d_revalidate with no inode but name = %s and dentry 0x%p\n",
 		      direntry->d_name.name, direntry));
 	}
+
+/*    unlock_kernel(); */
+
 	return isValid;
 }
 
@@ -285,7 +297,7 @@ static int cifs_d_revalidate(struct dentry *direntry, int flags)
 }     */
 
 struct dentry_operations cifs_dentry_ops = {
-	d_revalidate:cifs_d_revalidate,
-	/* d_delete:       cifs_d_delete,       */
+	d_revalidate:	 cifs_d_revalidate,
+/* d_delete:       cifs_d_delete,       *//* not needed except for debugging */
 	/* no need for d_hash, d_compare, d_release, d_iput ... yet. BB confirm this BB */
 };
