@@ -24,6 +24,8 @@
 #include <linux/stat.h>
 #include <linux/fcntl.h>
 #include <linux/version.h>
+#include <linux/pagemap.h>
+#include <linux/smp_lock.h>
 #include <asm/div64.h>
 #include "cifsfs.h"
 #include "cifspdu.h"
@@ -33,12 +35,15 @@
 #include "cifs_debug.h"
 #include "cifs_fs_sb.h"
 
-int cifs_open(struct inode *inode, struct file *file)
+int
+cifs_open(struct inode *inode, struct file *file)
 {
 	int rc = -EACCES;
 	int xid, oplock;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
+	struct cifsFileInfo *pCifsFile;
+	struct cifsInodeInfo *pCifsInode;
 	char *full_path = NULL;
 	int desiredAccess = 0x20197;
 	int disposition = FILE_OPEN;
@@ -47,7 +52,7 @@ int cifs_open(struct inode *inode, struct file *file)
 	xid = GetXid();
 
 	cFYI(1, (" inode = 0x%p file flags are %x", inode, file->f_flags));
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_path_from_dentry(file->f_dentry);
@@ -59,26 +64,9 @@ int cifs_open(struct inode *inode, struct file *file)
 	else if ((file->f_flags & O_ACCMODE) == O_RDWR)
 		desiredAccess = GENERIC_ALL;
 
-/*	if ((file->f_flags & O_ACCMODE) == O_RDONLY)   
-        desiredAccess =
-		    FILE_READ_DATA | FILE_READ_ATTRIBUTES | FILE_EXECUTE |
-		    READ_CONTROL;
-	else if ((file->f_flags & O_ACCMODE) == O_WRONLY)
-		desiredAccess =
-		    FILE_WRITE_DATA | FILE_WRITE_ATTRIBUTES |
-		    FILE_APPEND_DATA;
-	else if ((file->f_flags & O_ACCMODE) == O_RDWR)
-		desiredAccess =
-		    FILE_WRITE_DATA | FILE_READ_DATA |
-		    FILE_WRITE_ATTRIBUTES | FILE_READ_ATTRIBUTES |
-		    FILE_APPEND_DATA | FILE_EXECUTE | READ_CONTROL;  */
+/* BB check other flags carefully to find equivalent NTCreateX flags */
 
-/* BB go through these flags carefully to find equivalent NTCreateX flags */
 /*
-#define O_ACCMODE	   0003
-#define O_RDONLY	     00
-#define O_WRONLY	     01
-#define O_RDWR		     02
 #define O_CREAT		   0100	
 #define O_EXCL		   0200	
 #define O_NOCTTY	   0400	
@@ -95,7 +83,7 @@ int cifs_open(struct inode *inode, struct file *file)
 #define O_ATOMICLOOKUP	01000000 */
 
 	if (file->f_flags & O_CREAT)
-		disposition = FILE_OPEN_IF;	/* open if exists, create if does not exist */
+		disposition = FILE_OVERWRITE;
 
 	/* BB finish adding in oplock support BB */
 	if (oplockEnabled)
@@ -103,23 +91,41 @@ int cifs_open(struct inode *inode, struct file *file)
 	else
 		oplock = FALSE;
 
-
 	/* BB pass O_SYNC flag through on file attributes .. BB */
-	rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,
-			 0x3FF
-			 /* i.e. 777 dummy value, ignored for time being */
-			 , &netfid, &oplock, cifs_sb->local_nls);
+	rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,			 
+			 -1 /* i.e. dummy value, ignored for time being */,
+			 &netfid, &oplock, cifs_sb->local_nls);
 	if (rc) {
 		cFYI(1, ("\ncifs_open returned 0x%x ", rc));
 		cFYI(1, (" oplock: %d ", oplock));
 	} else {
 		file->private_data =
-		    kmalloc(sizeof(struct cifsFileInfo), GFP_KERNEL);
+		    kmalloc(sizeof (struct cifsFileInfo), GFP_KERNEL);
 		if (file->private_data) {
 			memset(file->private_data, 0,
-			       sizeof(struct cifsFileInfo));
-			((struct cifsFileInfo *) file->private_data)->
-			    netfid = netfid;
+			       sizeof (struct cifsFileInfo));
+			pCifsFile = (struct cifsFileInfo *) file->private_data;
+			pCifsFile->netfid = netfid;
+			pCifsFile->pid = current->pid;
+			pCifsFile->pfile = file; /* needed for writepage */
+			list_add(&pCifsFile->tlist,&pTcon->openFileList);
+			pCifsInode = CIFS_I(file->f_dentry->d_inode);
+			if(pCifsInode->openFileList.next)
+				list_add(&pCifsFile->flist,&pCifsInode->openFileList);
+			if(file->f_flags & O_CREAT) {           
+				/* time to set mode which we can not set earlier due
+				 to problems creating new read-only files */
+				if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)                
+					CIFSSMBUnixSetPerms(xid, pTcon, full_path, inode->i_mode,
+						0xFFFFFFFFFFFFFFFF,  
+						0xFFFFFFFFFFFFFFFF,
+						cifs_sb->local_nls);
+				else {/* BB implement via Windows security descriptors */
+			/* eg CIFSSMBWinSetPerms(xid,pTcon,full_path,mode,-1,-1,local_nls);*/
+			/* in the meantime could set r/o dos attribute when perms are eg:
+					mode & 0222 == 0 */
+				}
+			}
 		}
 	}
 
@@ -129,7 +135,8 @@ int cifs_open(struct inode *inode, struct file *file)
 	return rc;
 }
 
-int cifs_close(struct inode *inode, struct file *file)
+int
+cifs_close(struct inode *inode, struct file *file)
 {
 	int rc = 0;
 	int xid;
@@ -142,10 +149,12 @@ int cifs_close(struct inode *inode, struct file *file)
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
-
 	if (pSMBFile) {
+		if(pSMBFile->flist.next)
+			list_del(&pSMBFile->flist);
+		list_del(&pSMBFile->tlist);
 		rc = CIFSSMBClose(xid, pTcon, pSMBFile->netfid);
 		kfree(file->private_data);
 		file->private_data = NULL;
@@ -154,10 +163,10 @@ int cifs_close(struct inode *inode, struct file *file)
 
 	FreeXid(xid);
 	return rc;
-
 }
 
-int cifs_closedir(struct inode *inode, struct file *file)
+int
+cifs_closedir(struct inode *inode, struct file *file)
 {
 	int rc = 0;
 	int xid;
@@ -177,8 +186,8 @@ int cifs_closedir(struct inode *inode, struct file *file)
 	return rc;
 }
 
-
-int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
+int
+cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 {
 	int rc, xid;
 	__u32 lockType = LOCKING_ANDX_LARGE_FILES;
@@ -203,7 +212,7 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 	if (pfLock->fl_flags & FL_FLOCK)
 		cFYI(1, ("\nFlock "));
 	if (pfLock->fl_flags & FL_BROKEN)
-		cFYI(1, ("\nBroken flock emulation "));
+		cFYI(1, ("\nBroken flock emulation"));
 	if (pfLock->fl_flags & FL_ACCESS)
 		cFYI(1, ("\nProcess suspended by mandatory locking "));
 	if (pfLock->fl_flags & FL_LEASE)
@@ -231,19 +240,15 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 	} else
 		cFYI(1, ("\nUnknown type of lock "));
 
-	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
+	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
 	pTcon = cifs_sb->tcon;
 
-
 	if (file->private_data == NULL) {
 		FreeXid(xid);
 		return -EBADF;
 	}
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,9))
-	if (cmd == F_GETLK) {
-#else
+
 	if (IS_GETLK(cmd)) {
-#endif
 		rc = CIFSSMBLock(xid, pTcon,
 				 ((struct cifsFileInfo *) file->
 				  private_data)->netfid,
@@ -255,8 +260,7 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 					 ((struct cifsFileInfo *) file->
 					  private_data)->netfid,
 					 length,
-					 pfLock->fl_start,
-					 1 /* numUnlock */ ,
+					 pfLock->fl_start, 1 /* numUnlock */ ,
 					 0 /* numLock */ , lockType,
 					 0 /* wait flag */ );
 			pfLock->fl_type = F_UNLCK;
@@ -268,7 +272,7 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 
 		} else {
 			/* if rc == ERR_SHARING_VIOLATION ? */
-			rc = 0;	/* do not change lock type to unlock since the range is in use */
+			rc = 0;	/* do not change lock type to unlock since range in use */
 		}
 
 		FreeXid(xid);
@@ -284,9 +288,9 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 	return rc;
 }
 
-
-ssize_t cifs_write(struct file * file, const char *write_data,
-		   size_t write_size, loff_t * poffset)
+ssize_t
+cifs_write(struct file * file, const char *write_data,
+	   size_t write_size, loff_t * poffset)
 {
 	int rc = 0;
 	int bytes_written = 0;
@@ -294,9 +298,10 @@ ssize_t cifs_write(struct file * file, const char *write_data,
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
 	int xid, long_op;
+
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
+	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
 	pTcon = cifs_sb->tcon;
 
 	/*cFYI(1,
@@ -331,31 +336,164 @@ ssize_t cifs_write(struct file * file, const char *write_data,
 			}
 		} else
 			*poffset += bytes_written;
-		long_op = FALSE;	/* subsequent writes should be fast - 15 seconds is plenty of time */
+		long_op = FALSE; /* subsequent writes fast - 15 seconds is plenty */
 	}
-	file->f_dentry->d_inode->i_ctime = file->f_dentry->d_inode->i_mtime = CURRENT_TIME;	/* BB is this right? */
+	file->f_dentry->d_inode->i_ctime = file->f_dentry->d_inode->i_mtime = CURRENT_TIME;
 	if (bytes_written > 0) {
 		if (*poffset > file->f_dentry->d_inode->i_size)
 			file->f_dentry->d_inode->i_size = *poffset;
 	}
 	mark_inode_dirty_sync(file->f_dentry->d_inode);
+	FreeXid(xid);
 	return total_written;
 }
 
-int cifs_fsync(struct file *file, struct dentry *dentry, int datasync)
+static int
+cifs_partialpagewrite(struct page *page,unsigned from, unsigned to)
+{
+	struct address_space *mapping = page->mapping;
+	loff_t offset = (loff_t)page->index << PAGE_CACHE_SHIFT;
+	char * write_data = kmap(page);
+	int rc = -EFAULT;
+	int bytes_written = 0;
+	struct cifs_sb_info *cifs_sb;
+	struct cifsTconInfo *pTcon;
+	struct inode *inode = page->mapping->host;
+	struct cifsInodeInfo *cifsInode;
+	struct cifsFileInfo *open_file = NULL;
+	int xid;
+
+	xid = GetXid();
+
+	cifs_sb = CIFS_SB(inode->i_sb);
+	pTcon = cifs_sb->tcon;
+
+	/* figure out which file struct to use 
+	if (file->private_data == NULL) {
+	   FreeXid(xid);
+	   return -EBADF;
+	}     
+	 */
+	if (!mapping) {
+		FreeXid(xid);
+		return -EFAULT;
+	} else if(!mapping->host) {
+		FreeXid(xid);
+		return -EFAULT;
+	}
+
+	if((to > PAGE_CACHE_SIZE) || (from > to))
+		return -EIO;
+
+        offset += (loff_t)from;
+	write_data += from;
+
+	cifsInode = CIFS_I(mapping->host);
+	if(!list_empty(&(cifsInode->openFileList))) {            
+		open_file = list_entry(cifsInode->openFileList.next,
+					   struct cifsFileInfo, flist);           
+	/* We could check if file is open for writing first */
+		if(open_file->pfile)
+			bytes_written = cifs_write(open_file->pfile, write_data,
+					   to-from, &offset);
+		/* Does mm or vfs already set times? */
+		inode->i_atime = inode->i_mtime = CURRENT_TIME;	
+		if ((bytes_written > 0) && (offset)) {
+			rc = 0;
+			if (offset > inode->i_size)
+				inode->i_size = offset;
+		} else if(bytes_written < 0) {
+			rc = bytes_written;
+		}
+		mark_inode_dirty_sync(inode);
+	} else {
+		cFYI(1,("\nNo open files to get file handle from"));
+		rc = -EIO;
+	}
+
+	FreeXid(xid);
+	return rc;
+}
+
+static int
+cifs_writepage(struct page* page)
+{
+	int rc = -EFAULT;
+	int xid;
+
+	xid = GetXid();
+	page_cache_get(page);
+	rc = cifs_partialpagewrite(page,0,PAGE_CACHE_SIZE);
+	/* insert call to SetPageToUpdate like function here? */
+	unlock_page(page);
+	page_cache_release(page);	
+	FreeXid(xid);
+	return rc;
+}
+
+static int
+cifs_commit_write(struct file *file, struct page *page, unsigned offset,
+		  unsigned to)
+{
+	int xid,rc;
+	xid = GetXid();
+
+	if(offset > to)
+		return -EIO;
+	rc = cifs_partialpagewrite(page,offset,to);
+
+	FreeXid(xid);
+	return rc;
+}
+
+static int
+cifs_prepare_write(struct file *file, struct page *page, unsigned offset,
+		   unsigned to)
+{
+	return 0;	/* eventually add code to flush any incompatible requests */
+}
+
+int
+cifs_fsync(struct file *file, struct dentry *dentry, int datasync)
 {
 	int xid;
 	int rc = 0;
 	xid = GetXid();
 	/* BB fill in code to flush write behind buffers - when we start supporting oplock */
-	cFYI(1,
-	     (" name: %s datasync: 0x%x ", dentry->d_name.name, datasync));
+	cFYI(1, (" name: %s datasync: 0x%x ", dentry->d_name.name, datasync));
 	FreeXid(xid);
 	return rc;
 }
 
-ssize_t cifs_read(struct file * file, char *read_data, size_t read_size,
-		  loff_t * poffset)
+static int
+cifs_sync_page(struct page *page)
+{
+	struct address_space *mapping;
+	struct inode *inode;
+	unsigned long index = page->index;
+	unsigned int rpages = 0;
+	int rc = 0;
+
+	mapping = page->mapping;
+	if (!mapping)
+		return 0;
+	inode = mapping->host;
+	if (!inode)
+		return 0;
+
+/*	fill in rpages then 
+    result = cifs_pagein_inode(inode, index, rpages); *//* BB finish */
+
+	cFYI(1, ("\nrpages is %d for sync page of Index %ld ", rpages, index));
+
+	if (rc < 0)
+		return rc;
+	return 0;
+}
+
+ssize_t
+cifs_read(struct file * file, char *read_data, size_t read_size,
+	  loff_t * poffset)
 {
 	int rc = -EACCES;
 	int bytes_read = 0;
@@ -365,7 +503,7 @@ ssize_t cifs_read(struct file * file, char *read_data, size_t read_size,
 	int xid;
 
 	xid = GetXid();
-	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
+	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
 	pTcon = cifs_sb->tcon;
 
 	if (file->private_data == NULL) {
@@ -373,8 +511,7 @@ ssize_t cifs_read(struct file * file, char *read_data, size_t read_size,
 		return -EBADF;
 	}
 
-	for (total_read = 0; read_size > total_read;
-	     total_read += bytes_read) {
+	for (total_read = 0; read_size > total_read; total_read += bytes_read) {
 		rc = CIFSSMBRead(xid, pTcon,
 				 ((struct cifsFileInfo *) file->
 				  private_data)->netfid,
@@ -395,11 +532,77 @@ ssize_t cifs_read(struct file * file, char *read_data, size_t read_size,
 	return total_read;
 }
 
-void fill_in_inode(struct inode *tmp_inode,
-		   FILE_DIRECTORY_INFO * pfindData, int *pobject_type)
+int
+cifs_file_mmap(struct file * file, struct vm_area_struct * vma)
 {
-	struct cifsInodeInfo *cifsInfo =
-	    (struct cifsInodeInfo *) &(tmp_inode->u);
+	struct dentry * dentry = file->f_dentry;
+	int	rc, xid;
+
+	xid = GetXid();
+	rc = cifs_revalidate(dentry);
+	if (rc) {
+		cFYI(1,("Validation prior to mmap failed, error=%d\n", rc));
+		FreeXid(xid);
+		return rc;
+	}
+	rc = generic_file_mmap(file, vma);
+	FreeXid(xid);
+	return rc;
+}
+
+static int
+cifs_readpage(struct file *file, struct page *page)
+{
+	loff_t offset = (loff_t)page->index << PAGE_CACHE_SHIFT;
+	char * read_data;
+	int rc = -EACCES;
+	struct cifs_sb_info *cifs_sb;
+	struct cifsTconInfo *pTcon;
+	int xid;
+
+	xid = GetXid();
+	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
+	pTcon = cifs_sb->tcon;
+
+	if (file->private_data == NULL) {
+		FreeXid(xid);
+		return -EBADF;
+	}
+	page_cache_get(page);
+	read_data = kmap(page);
+	/* for reads over a certain size we could initiate async read ahead */
+
+	rc = cifs_read(file, read_data, PAGE_CACHE_SIZE, &offset);
+
+	if (rc < 0)
+		goto io_error;
+	else {
+		cFYI(1,("\nBytes read %d ",rc));
+	}
+
+	file->f_dentry->d_inode->i_atime = CURRENT_TIME;
+
+	if(PAGE_CACHE_SIZE > rc) {
+		memset(read_data+rc, 0, PAGE_CACHE_SIZE - rc);
+	}
+	flush_dcache_page(page);
+	SetPageUptodate(page);
+	rc = 0;
+
+io_error:
+	kunmap(page);
+	unlock_page(page);
+
+	page_cache_release(page);
+	FreeXid(xid);
+	return rc;
+}
+
+void
+fill_in_inode(struct inode *tmp_inode,
+	      FILE_DIRECTORY_INFO * pfindData, int *pobject_type)
+{
+	struct cifsInodeInfo *cifsInfo = CIFS_I(tmp_inode);
 	pfindData->ExtFileAttributes =
 	    le32_to_cpu(pfindData->ExtFileAttributes);
 	pfindData->AllocationSize = le64_to_cpu(pfindData->AllocationSize);
@@ -410,11 +613,11 @@ void fill_in_inode(struct inode *tmp_inode,
 
 	/* Linux can not store file creation time unfortunately so we ignore it */
 	tmp_inode->i_atime =
-	    le64_to_cpu(cifs_NTtimeToUnix(pfindData->LastAccessTime));
+	    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastAccessTime));
 	tmp_inode->i_mtime =
-	    le64_to_cpu(cifs_NTtimeToUnix(pfindData->LastWriteTime));
+	    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastWriteTime));
 	tmp_inode->i_ctime =
-	    le64_to_cpu(cifs_NTtimeToUnix(pfindData->ChangeTime));
+	    cifs_NTtimeToUnix(le64_to_cpu(pfindData->ChangeTime));
 	/* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
 	tmp_inode->i_mode = S_IALLUGO & ~(S_ISUID | S_IXGRP);	/* 2767 perms - indicate mandatory locking */
 	cFYI(0,
@@ -430,17 +633,14 @@ void fill_in_inode(struct inode *tmp_inode,
 	} else {
 		*pobject_type = DT_REG;
 		tmp_inode->i_mode |= S_IFREG;
-	}			/* BB add code here - to validate if device or weird share type or odd device type? BB */
+	}/* could add code here - to validate if device or weird share type? */
 
-	/* can not fill in nlink here as we can in both qpathinfo versions and in unix search */
+	/* can not fill in nlink here as in qpathinfo version and in Unx search */
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
 	    do_div(pfindData->AllocationSize, tmp_inode->i_blksize);
-	cFYI(1,
-	     ("\nFinddata alloc size (from smb) %lld",
-	      pfindData->AllocationSize));
 	if (pfindData->AllocationSize < pfindData->EndOfFile)
-		cFYI(1, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
+		cFYI(1, ("\nServer inconsistency Error: it says allocation size less than end of file "));
 	cFYI(1,
 	     ("\nCIFS FFIRST: Size %ld and blocks %ld and blocksize %ld",
 	      (unsigned long) tmp_inode->i_size, tmp_inode->i_blocks,
@@ -463,27 +663,21 @@ void fill_in_inode(struct inode *tmp_inode,
 	}
 }
 
-/* BB make endian neutral */
-void unix_fill_in_inode(struct inode *tmp_inode,
-			FILE_UNIX_INFO * pfindData, int *pobject_type)
+void
+unix_fill_in_inode(struct inode *tmp_inode,
+		   FILE_UNIX_INFO * pfindData, int *pobject_type)
 {
-	struct cifsInodeInfo *cifsInfo =
-	    (struct cifsInodeInfo *) &(tmp_inode->u);
+	struct cifsInodeInfo *cifsInfo = CIFS_I(tmp_inode);
 	cifsInfo->time = jiffies;
 	atomic_inc(&cifsInfo->inUse);
 
 	tmp_inode->i_atime =
-	    le64_to_cpu(cifs_NTtimeToUnix(pfindData->LastAccessTime));
+	    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastAccessTime));
 	tmp_inode->i_mtime =
-	    le64_to_cpu(cifs_NTtimeToUnix
-			(pfindData->LastModificationTime));
+	    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastModificationTime));
 	tmp_inode->i_ctime =
-	    le64_to_cpu(cifs_NTtimeToUnix(pfindData->LastStatusChange));
+	    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastStatusChange));
 
-	/* tmp_inode->i_mtime =
-	   cifs_NTtimeToUnix(pfindData->LastModificationTime);
-	   tmp_inode->i_ctime =
-	   cifs_NTtimeToUnix(pfindData->LastStatusChange); */
 	tmp_inode->i_mode = le64_to_cpu(pfindData->Permissions);
 	pfindData->Type = le32_to_cpu(pfindData->Type);
 	if (pfindData->Type == UNIX_FILE) {
@@ -518,10 +712,6 @@ void unix_fill_in_inode(struct inode *tmp_inode,
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
 	    do_div(pfindData->NumOfBytes, tmp_inode->i_blksize);
-	cFYI(0, ("\nFinddata alloc size (from smb) %lld", pfindData->NumOfBytes));	/* BB remove */
-	if (pfindData->NumOfBytes < pfindData->EndOfFile)
-		cFYI(0, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
-	cFYI(1, ("\nCIFS FFIRST: Size %ld and blocks %ld ", (unsigned long) tmp_inode->i_size, tmp_inode->i_blocks));	/* BB remove */
 	if (S_ISREG(tmp_inode->i_mode)) {
 		cFYI(1, (" File inode "));
 		tmp_inode->i_op = &cifs_file_inode_ops;
@@ -541,16 +731,17 @@ void unix_fill_in_inode(struct inode *tmp_inode,
 	}
 }
 
-void construct_dentry(struct qstr *qstring, struct file *file,
-		      struct inode **ptmp_inode,
-		      struct dentry **pnew_dentry)
+void
+construct_dentry(struct qstr *qstring, struct file *file,
+		 struct inode **ptmp_inode, struct dentry **pnew_dentry)
 {
 	struct dentry *tmp_dentry;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
+	struct cifsInodeInfo *pCifsInode;
 
 	cFYI(1, ("\nFor %s ", qstring->name));
-	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
+	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
 	pTcon = cifs_sb->tcon;
 
 	qstring->hash = full_name_hash(qstring->name, qstring->len);
@@ -562,6 +753,11 @@ void construct_dentry(struct qstr *qstring, struct file *file,
 	} else {
 		tmp_dentry = d_alloc(file->f_dentry, qstring);
 		*ptmp_inode = new_inode(file->f_dentry->d_sb);
+		if(*ptmp_inode) {
+			pCifsInode = CIFS_I(*ptmp_inode);
+			pCifsInode->clientCanCache = 0;
+			INIT_LIST_HEAD(&pCifsInode->openFileList);
+		}
 		cFYI(1, ("\nAlloc new inode %p ", *ptmp_inode));
 		tmp_dentry->d_op = &cifs_dentry_ops;
 		cFYI(1, (" instantiate dentry 0x%p with inode 0x%p ",
@@ -577,8 +773,9 @@ void construct_dentry(struct qstr *qstring, struct file *file,
 	*pnew_dentry = tmp_dentry;
 }
 
-void cifs_filldir(struct qstr *pqstring, FILE_DIRECTORY_INFO * pfindData,
-		  struct file *file, filldir_t filldir, void *direntry)
+void
+cifs_filldir(struct qstr *pqstring, FILE_DIRECTORY_INFO * pfindData,
+	     struct file *file, filldir_t filldir, void *direntry)
 {
 	struct inode *tmp_inode;
 	struct dentry *tmp_dentry;
@@ -595,9 +792,10 @@ void cifs_filldir(struct qstr *pqstring, FILE_DIRECTORY_INFO * pfindData,
 	dput(tmp_dentry);
 }
 
-void cifs_filldir_unix(struct qstr *pqstring,
-		       FILE_UNIX_INFO * pUnixFindData, struct file *file,
-		       filldir_t filldir, void *direntry)
+void
+cifs_filldir_unix(struct qstr *pqstring,
+		  FILE_UNIX_INFO * pUnixFindData, struct file *file,
+		  filldir_t filldir, void *direntry)
 {
 	struct inode *tmp_inode;
 	struct dentry *tmp_dentry;
@@ -614,10 +812,8 @@ void cifs_filldir_unix(struct qstr *pqstring,
 	dput(tmp_dentry);
 }
 
-
-
-
-int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
+int
+cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 {
 	int rc = 0;
 	int xid, i;
@@ -640,14 +836,12 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 	data = kmalloc(4096, GFP_KERNEL);
 	pfindData = (FILE_DIRECTORY_INFO *) data;
 
-	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
+	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_wildcard_path_from_dentry(file->f_dentry);
 
-	cFYI(1,
-	     ("\nFull path: %s start at: %lld ", full_path, file->f_pos));
-
+	cFYI(1, ("\nFull path: %s start at: %lld ", full_path, file->f_pos));
 
 	switch ((int) file->f_pos) {
 	case 0:
@@ -662,8 +856,7 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 	case 1:
 		if (filldir
 		    (direntry, "..", 2, file->f_pos,
-		     file->f_dentry->d_parent->d_inode->i_ino,
-		     DT_DIR) < 0) {
+		     file->f_dentry->d_parent->d_inode->i_ino, DT_DIR) < 0) {
 			cERROR(1, ("\nFilldir for parent dir failed "));
 			break;
 		}
@@ -681,14 +874,12 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 		if (rc == 0) {
 			searchHandle = findParms.SearchHandle;
 			file->private_data =
-			    kmalloc(sizeof(struct cifsFileInfo),
-				    GFP_KERNEL);
+			    kmalloc(sizeof (struct cifsFileInfo), GFP_KERNEL);
 			if (file->private_data) {
 				memset(file->private_data, 0,
-				       sizeof(struct cifsFileInfo));
+				       sizeof (struct cifsFileInfo));
 				cifsFile =
-				    (struct cifsFileInfo *) file->
-				    private_data;
+				    (struct cifsFileInfo *) file->private_data;
 				cifsFile->netfid = searchHandle;
 			}
 
@@ -703,16 +894,14 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 						pfindData->FileNameLength =
 						    cifs_strfromUCS_le
 						    (pfindData->FileName,
-						     (UniChar *)
+						     (wchar_t *)
 						     pfindData->FileName,
 						     (pfindData->
 						      FileNameLength) / 2,
 						     cifs_sb->local_nls);
-					qstring.len =
-					    pfindData->FileNameLength;
+					qstring.len = pfindData->FileNameLength;
 					if (((qstring.len != 1)
-					     || (pfindData->FileName[0] !=
-						 '.'))
+					     || (pfindData->FileName[0] != '.'))
 					    && ((qstring.len != 2)
 						|| (pfindData->
 						    FileName[0] != '.')
@@ -731,7 +920,7 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 						qstring.len =
 						    cifs_strfromUCS_le
 						    (pfindDataUnix->
-						     FileName, (UniChar *)
+						     FileName, (wchar_t *)
 						     pfindDataUnix->
 						     FileName,
 						     MAX_PATHCONF,
@@ -776,8 +965,7 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 			     ("\nReaddir on closed srch, pos = %lld",
 			      file->f_pos));
 		} else {
-			cifsFile =
-			    (struct cifsFileInfo *) file->private_data;
+			cifsFile = (struct cifsFileInfo *) file->private_data;
 			if (cifsFile->endOfSearch) {
 				rc = 0;
 				cFYI(1, ("\nEnd of search "));
@@ -792,8 +980,7 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 			      findNextParms.SearchCount,
 			      findNextParms.EndofSearch));
 			if ((rc == 0) && (findNextParms.SearchCount != 0)) {
-				for (i = 0; i < findNextParms.SearchCount;
-				     i++) {
+				for (i = 0; i < findNextParms.SearchCount; i++) {
 					pfindData->FileNameLength =
 					    le32_to_cpu(pfindData->
 							FileNameLength);
@@ -805,7 +992,7 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 							    cifs_strfromUCS_le
 							    (pfindData->
 							     FileName,
-							     (UniChar *)
+							     (wchar_t *)
 							     pfindData->
 							     FileName,
 							     (pfindData->
@@ -814,16 +1001,13 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 							     cifs_sb->
 							     local_nls);
 						qstring.len =
-						    pfindData->
-						    FileNameLength;
+						    pfindData->FileNameLength;
 						if (((qstring.len != 1)
 						     || (pfindData->
-							 FileName[0] !=
-							 '.'))
+							 FileName[0] != '.'))
 						    && ((qstring.len != 2)
 							|| (pfindData->
-							    FileName[0] !=
-							    '.')
+							    FileName[0] != '.')
 							|| (pfindData->
 							    FileName[1] !=
 							    '.'))) {
@@ -843,7 +1027,7 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 							    cifs_strfromUCS_le
 							    (pfindDataUnix->
 							     FileName,
-							     (UniChar *)
+							     (wchar_t *)
 							     pfindDataUnix->
 							     FileName,
 							     MAX_PATHCONF,
@@ -858,12 +1042,10 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 
 						if (((qstring.len != 1)
 						     || (pfindDataUnix->
-							 FileName[0] !=
-							 '.'))
+							 FileName[0] != '.'))
 						    && ((qstring.len != 2)
 							|| (pfindDataUnix->
-							    FileName[0] !=
-							    '.')
+							    FileName[0] != '.')
 							|| (pfindDataUnix->
 							    FileName[1] !=
 							    '.'))) {
@@ -897,3 +1079,11 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 
 	return rc;
 }
+
+struct address_space_operations cifs_addr_ops = {
+	readpage:cifs_readpage,
+/*	sync_page:cifs_sync_page,*/
+	writepage:cifs_writepage,
+	prepare_write:cifs_prepare_write,
+	commit_write:cifs_commit_write
+};
