@@ -28,16 +28,18 @@
 #include "cifs_debug.h"
 #include "cifs_fs_sb.h"
 
-int cifs_get_inode_info_unix(struct inode **pinode,
-			     const unsigned char *search_path,
-			     struct super_block *sb)
+int
+cifs_get_inode_info_unix(struct inode **pinode,
+			 const unsigned char *search_path,
+			 struct super_block *sb)
 {
 	int xid;
+	int newinode = FALSE;
 	int rc = 0;
 	FILE_UNIX_BASIC_INFO findData;
 	struct cifsTconInfo *pTcon;
 	struct inode *inode;
-	struct cifs_sb_info *cifs_sb = (struct cifs_sb_info *) &sb->u;
+	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
 	char *tmp_path;
 
 	xid = GetXid();
@@ -51,9 +53,9 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 	if (rc) {
 		if (rc == -EREMOTE) {
 /* rc = *//* CIFSGetDFSRefer(xid, pTcon->ses, search_path,
-   &referrals,
-   &num_referrals,
-   cifs_sb->local_nls); */
+	&referrals,
+	&num_referrals,
+	cifs_sb->local_nls); */
 			tmp_path =
 			    kmalloc(strnlen
 				    (pTcon->treeName,
@@ -64,8 +66,8 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 				FreeXid(xid);
 				return -ENOMEM;
 			}
-
-			strncpy(tmp_path, pTcon->treeName, MAX_TREE_SIZE);	/* have to skip first of the double backslash of UNC name */
+        /* have to skip first of the double backslash of UNC name */
+			strncpy(tmp_path, pTcon->treeName, MAX_TREE_SIZE);	
 			strncat(tmp_path, search_path, MAX_PATHCONF);
 			rc = connect_to_dfs_path(xid, pTcon->ses,
 						 /* treename + */ tmp_path,
@@ -85,26 +87,32 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 		if (*pinode == NULL) {
 			*pinode = new_inode(sb);
 			cFYI(1, ("\nAlloc new inode %p ", *pinode));
+			if(*pinode)  {
+				newinode = TRUE;
+			}
 		}
 		inode = *pinode;
-/*        new_inode = iget(parent_dir_inode->i_sb, findData.IndexNumber); *//* index number not reliable in response data */
+/*      new_inode = iget(parent_dir_inode->i_sb, findData.IndexNumber); */
+        /* index number not reliable in response data */
 
-		cifsInfo = (struct cifsInodeInfo *) &(inode->u);
+		cifsInfo = CIFS_I(inode);
 
 		cFYI(1, ("\nOld time %ld ", cifsInfo->time));
 		cifsInfo->time = jiffies;
 		cFYI(1, (" New time %ld ", cifsInfo->time));
+		if(newinode == TRUE) {
+			cifsInfo->clientCanCache = 0;
+			INIT_LIST_HEAD(&cifsInfo->openFileList);
+		}
 		atomic_inc(&cifsInfo->inUse);	/* inc on every refresh of inode */
 
 		inode->i_atime =
-		    le64_to_cpu(cifs_NTtimeToUnix
-				(findData.LastAccessTime));
+		    cifs_NTtimeToUnix(le64_to_cpu(findData.LastAccessTime));
 		inode->i_mtime =
-		    le64_to_cpu(cifs_NTtimeToUnix
+		    cifs_NTtimeToUnix(le64_to_cpu
 				(findData.LastModificationTime));
 		inode->i_ctime =
-		    le64_to_cpu(cifs_NTtimeToUnix
-				(findData.LastStatusChange));
+		    cifs_NTtimeToUnix(le64_to_cpu(findData.LastStatusChange));
 		inode->i_mode = le64_to_cpu(findData.Permissions);
 		findData.Type = le32_to_cpu(findData.Type);
 		if (findData.Type == UNIX_FILE) {
@@ -130,13 +138,9 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 		inode->i_size = findData.EndOfFile;
 		inode->i_blksize =
 		    (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
-		inode->i_blocks =
-		    do_div(findData.NumOfBytes, inode->i_blksize);
-		cFYI(1,
-		     ("\nFinddata alloc size (from smb) %lld",
-		      findData.NumOfBytes));
+		inode->i_blocks = do_div(findData.NumOfBytes, inode->i_blksize);
 		if (findData.NumOfBytes < findData.EndOfFile)
-			cFYI(1, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
+			cFYI(1, ("\nServer inconsistency Error: it says allocation size less than end of file "));
 		cFYI(1,
 		     ("\nCIFS FFIRST: Size %ld and blocks %ld ",
 		      (unsigned long) inode->i_size, inode->i_blocks));
@@ -144,6 +148,7 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 			cFYI(1, (" File inode "));
 			inode->i_op = &cifs_file_inode_ops;
 			inode->i_fop = &cifs_file_ops;
+			inode->i_data.a_ops = &cifs_addr_ops;
 		} else if (S_ISDIR(inode->i_mode)) {
 			cFYI(1, (" Directory inode"));
 			inode->i_op = &cifs_dir_inode_ops;
@@ -162,16 +167,17 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 	return rc;
 }
 
-int cifs_get_inode_info(struct inode **pinode,
-			const unsigned char *search_path,
-			struct super_block *sb)
+int
+cifs_get_inode_info(struct inode **pinode,
+		    const unsigned char *search_path, struct super_block *sb)
 {
 	int xid;
 	int rc = 0;
+	int newinode = FALSE;
 	FILE_ALL_INFO findData;
 	struct cifsTconInfo *pTcon;
 	struct inode *inode;
-	struct cifs_sb_info *cifs_sb = (struct cifs_sb_info *) &sb->u;
+	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
 	char *tmp_path;
 
 	xid = GetXid();
@@ -186,9 +192,9 @@ int cifs_get_inode_info(struct inode **pinode,
 		if (rc == -EREMOTE) {
 			/* BB add call to new func rc = GetDFSReferral(); */
 /* rc = *//* CIFSGetDFSRefer(xid, pTcon->ses, search_path,
-   &referrals,
-   &num_referrals,
-   cifs_sb->local_nls); */
+	&referrals,
+	&num_referrals,
+	cifs_sb->local_nls); */
 			tmp_path =
 			    kmalloc(strnlen
 				    (pTcon->treeName,
@@ -218,23 +224,29 @@ int cifs_get_inode_info(struct inode **pinode,
 		if (*pinode == NULL) {
 			*pinode = new_inode(sb);
 			cFYI(1, ("\nAlloc new inode %p ", *pinode));
+			if(*pinode) {
+				newinode = TRUE;
+			}
 		}
 
 		inode = *pinode;
-		cifsInfo = (struct cifsInodeInfo *) &(inode->u);
+		cifsInfo = CIFS_I(inode);
 		findData.Attributes = le32_to_cpu(findData.Attributes);
 		cifsInfo->cifsAttrs = findData.Attributes;
 		cFYI(1, ("\nOld time %ld ", cifsInfo->time));
 		cifsInfo->time = jiffies;
 		cFYI(1, (" New time %ld ", cifsInfo->time));
+		if(newinode == TRUE) {
+			cifsInfo->clientCanCache = 0;
+			INIT_LIST_HEAD(&cifsInfo->openFileList);
+		}
 		atomic_inc(&cifsInfo->inUse);	/* inc on every refresh of inode */
 
 		inode->i_blksize =
 		    (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
 		/* Linux can not store file creation time unfortunately so we ignore it */
 		inode->i_atime =
-		    cifs_NTtimeToUnix(le64_to_cpu
-				      (findData.LastAccessTime));
+		    cifs_NTtimeToUnix(le64_to_cpu(findData.LastAccessTime));
 		inode->i_mtime =
 		    cifs_NTtimeToUnix(le64_to_cpu(findData.LastWriteTime));
 		inode->i_ctime =
@@ -243,20 +255,20 @@ int cifs_get_inode_info(struct inode **pinode,
 		/* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
 		inode->i_mode = S_IALLUGO & ~(S_ISUID | S_IXGRP);	/* 2767 perms indicate mandatory locking - will override for dirs later */
 		cFYI(0,
-		     ("\nAttributes came in as 0x%x\n",
-		      findData.Attributes));
-		if (findData.Attributes & ATTR_REPARSE) {	/* BB can IFLNK be set, as it basically is on windows, with IFREG or IFDIR too */
+		     ("\nAttributes came in as 0x%x\n", findData.Attributes));
+		if (findData.Attributes & ATTR_REPARSE) {	
+   /* Can IFLNK be set as it basically is on windows with IFREG or IFDIR? */
 			inode->i_mode |= S_IFLNK;
 		} else if (findData.Attributes & ATTR_DIRECTORY) {
-			inode->i_mode = S_IRWXUGO;	/* override default perms since we do not do byte range locking on dirs */
-			inode->i_mode |= S_IFDIR;
+   /* override default perms since we do not do byte range locking on dirs */
+			inode->i_mode = S_IRWXUGO;	
+            inode->i_mode |= S_IFDIR;
 		} else {
 			inode->i_mode |= S_IFREG;
-			/* BB add code here - validate if device or weird share type or device type? BB */
+   /* BB add code here - validate if device or weird share or device type? */
 		}
 		inode->i_size = le64_to_cpu(findData.EndOfFile);
-		findData.AllocationSize =
-		    le64_to_cpu(findData.AllocationSize);
+		findData.AllocationSize = le64_to_cpu(findData.AllocationSize);
 		inode->i_blocks =
 		    do_div(findData.AllocationSize, inode->i_blksize);
 		cFYI(1,
@@ -264,12 +276,13 @@ int cifs_get_inode_info(struct inode **pinode,
 		      (unsigned long) inode->i_size, inode->i_blocks));
 		inode->i_nlink = le32_to_cpu(findData.NumberOfLinks);
 
-		/* BB fill in uid and gid here? with help from winbind? BB */
+		/* BB fill in uid and gid here? with help from winbind? */
 
 		if (S_ISREG(inode->i_mode)) {
 			cFYI(1, (" File inode "));
 			inode->i_op = &cifs_file_inode_ops;
 			inode->i_fop = &cifs_file_ops;
+			inode->i_data.a_ops = &cifs_addr_ops;
 		} else if (S_ISDIR(inode->i_mode)) {
 			cFYI(1, (" Directory inode "));
 			inode->i_op = &cifs_dir_inode_ops;
@@ -286,13 +299,13 @@ int cifs_get_inode_info(struct inode **pinode,
 	return rc;
 }
 
-
-void cifs_read_inode(struct inode *inode)
+void
+cifs_read_inode(struct inode *inode)
 {				/* gets root inode */
 
 	struct cifs_sb_info *cifs_sb;
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 
 	if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)
 		cifs_get_inode_info_unix(&inode, "", inode->i_sb);
@@ -300,7 +313,8 @@ void cifs_read_inode(struct inode *inode)
 		cifs_get_inode_info(&inode, "", inode->i_sb);
 }
 
-int cifs_unlink(struct inode *inode, struct dentry *direntry)
+int
+cifs_unlink(struct inode *inode, struct dentry *direntry)
 {
 	int rc = 0;
 	int xid;
@@ -309,12 +323,11 @@ int cifs_unlink(struct inode *inode, struct dentry *direntry)
 	char *full_path = NULL;
 	struct cifsInodeInfo *cifsInode;
 
-
 	cFYI(1, ("\n cifs_unlink, inode = 0x%p with ", inode));
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	/* BB Should we close the file if it is already open from our client? */
@@ -326,11 +339,11 @@ int cifs_unlink(struct inode *inode, struct dentry *direntry)
 	if (!rc) {
 		direntry->d_inode->i_nlink--;
 	}
-	cifsInode = (struct cifsInodeInfo *) &direntry->d_inode->u;
-	cifsInode->time = 0;	/* will force revalidate to go get info when needed */
+	cifsInode = CIFS_I(direntry->d_inode);
+	cifsInode->time = 0;	/* will force revalidate to get info when needed */
 	direntry->d_inode->i_ctime = inode->i_ctime = inode->i_mtime =
 	    CURRENT_TIME;
-	cifsInode = (struct cifsInodeInfo *) &inode->u;
+	cifsInode = CIFS_I(inode);
 	cifsInode->time = 0;	/* force revalidate of dir as well */
 
 	if (full_path)
@@ -339,7 +352,8 @@ int cifs_unlink(struct inode *inode, struct dentry *direntry)
 	return rc;
 }
 
-int cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
+int
+cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 {
 	int rc = 0;
 	int xid;
@@ -348,15 +362,15 @@ int cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 	char *full_path = NULL;
 	struct inode *newinode = NULL;
 
-	cFYI(1, ("In cifs_mkdir, inode = 0x%p\n", inode));
+	cFYI(1, ("In cifs_mkdir, mode = 0x%x inode = 0x%p\n", mode, inode));
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_path_from_dentry(direntry);
-	/* BB add processing for setting the equivalent of mode - e.g. via CreateX with ACLs */
+	/* BB add setting the equivalent of mode via CreateX w/ACLs */
 	rc = CIFSSMBMkDir(xid, pTcon, full_path, cifs_sb->local_nls);
 	if (rc) {
 		cFYI(1, ("\ncifs_mkdir returned 0x%x ", rc));
@@ -372,6 +386,15 @@ int cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 		direntry->d_op = &cifs_dentry_ops;
 		d_instantiate(direntry, newinode);
 		direntry->d_inode->i_nlink = 2;
+        if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)                
+            CIFSSMBUnixSetPerms(xid, pTcon, full_path, mode,
+                        0xFFFFFFFFFFFFFFFF,  
+                        0xFFFFFFFFFFFFFFFF,
+                        cifs_sb->local_nls);
+        else { /* BB to be implemented via Windows secrty descriptors*/
+        /* eg CIFSSMBWinSetPerms(xid,pTcon,full_path,mode,-1,-1,local_nls);*/
+        }
+
 	}
 	if (full_path)
 		kfree(full_path);
@@ -380,7 +403,8 @@ int cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 	return rc;
 }
 
-int cifs_rmdir(struct inode *inode, struct dentry *direntry)
+int
+cifs_rmdir(struct inode *inode, struct dentry *direntry)
 {
 	int rc = 0;
 	int xid;
@@ -393,7 +417,7 @@ int cifs_rmdir(struct inode *inode, struct dentry *direntry)
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_path_from_dentry(direntry);
@@ -406,8 +430,8 @@ int cifs_rmdir(struct inode *inode, struct dentry *direntry)
 		direntry->d_inode->i_nlink = 0;
 	}
 
-	cifsInode = (struct cifsInodeInfo *) &direntry->d_inode->u;
-	cifsInode->time = 0;	/* will force revalidate to go get info when needed */
+	cifsInode = CIFS_I(direntry->d_inode);
+	cifsInode->time = 0;	/* force revalidate to go get info when needed */
 	direntry->d_inode->i_ctime = inode->i_ctime = inode->i_mtime =
 	    CURRENT_TIME;
 
@@ -417,8 +441,9 @@ int cifs_rmdir(struct inode *inode, struct dentry *direntry)
 	return rc;
 }
 
-int cifs_rename(struct inode *source_inode, struct dentry *source_direntry,
-		struct inode *target_inode, struct dentry *target_direntry)
+int
+cifs_rename(struct inode *source_inode, struct dentry *source_direntry,
+	    struct inode *target_inode, struct dentry *target_direntry)
 {
 	char *fromName;
 	char *toName;
@@ -430,14 +455,15 @@ int cifs_rename(struct inode *source_inode, struct dentry *source_direntry,
 
 	xid = GetXid();
 
-	cifs_sb_target =
-	    (struct cifs_sb_info *) (&(target_inode->i_sb->u));
-	cifs_sb_source =
-	    (struct cifs_sb_info *) (&(source_inode->i_sb->u));
+	cifs_sb_target = CIFS_SB(target_inode->i_sb);
+	cifs_sb_source = CIFS_SB(source_inode->i_sb);
 	pTcon = cifs_sb_source->tcon;
 
-	if (pTcon != cifs_sb_target->tcon)
-		return -EXDEV;	/* actually could be allowed if same server but different share, BB fix eventually BB */
+	if (pTcon != cifs_sb_target->tcon) {    
+		return -EXDEV;	/* BB actually could be allowed if same server, but
+                     different share. Might eventually add support for this */
+        FreeXid(xid);
+    }
 
 	fromName = build_path_from_dentry(source_direntry);
 	toName = build_path_from_dentry(target_direntry);
@@ -449,10 +475,12 @@ int cifs_rename(struct inode *source_inode, struct dentry *source_direntry,
 	if (toName)
 		kfree(toName);
 
+    FreeXid(xid);
 	return rc;
 }
 
-int cifs_revalidate(struct dentry *direntry)
+int
+cifs_revalidate(struct dentry *direntry)
 {
 	int xid;
 	int rc = 0;
@@ -462,7 +490,7 @@ int cifs_revalidate(struct dentry *direntry)
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(direntry->d_sb->u));
+	cifs_sb = CIFS_SB(direntry->d_sb);
 
 	full_path = build_path_from_dentry(direntry);
 	cFYI(1,
@@ -471,9 +499,8 @@ int cifs_revalidate(struct dentry *direntry)
 	      direntry->d_inode->i_count.counter, direntry,
 	      direntry->d_time, jiffies));
 
-	cifsInode = (struct cifsInodeInfo *) &(direntry->d_inode->u);
-
-
+	cifsInode = CIFS_I(direntry->d_inode);
+/* BB add check - do not need to revalidate oplocked files */
 	if ((time_before(jiffies, cifsInode->time + HZ))
 	    && (direntry->d_inode->i_nlink == 1)) {
 		cFYI(1, (" Do not need to revalidate "));
@@ -499,18 +526,21 @@ int cifs_revalidate(struct dentry *direntry)
 	return rc;
 }
 
-void cifs_truncate_file(struct inode *inode)
+void
+cifs_truncate_file(struct inode *inode)
 {				/* BB remove - may not need this function after all BB */
 	int xid;
 	int rc = 0;
+	struct cifsFileInfo *open_file = NULL;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
+	struct cifsInodeInfo *cifsInode;
 	struct dentry *dirent;
-	char *full_path = NULL;
+	char *full_path = NULL;   
 
 	xid = GetXid();
 
-	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
+	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	if (list_empty(&inode->i_dentry)) {
@@ -523,14 +553,24 @@ void cifs_truncate_file(struct inode *inode)
 	dirent = list_entry(inode->i_dentry.next, struct dentry, d_alias);
 	if (dirent) {
 		full_path = build_path_from_dentry(dirent);
-		rc = CIFSSMBSetEOF(xid, pTcon, full_path, inode->i_size,
+		rc = CIFSSMBSetEOF(xid, pTcon, full_path, inode->i_size,FALSE,
 				   cifs_sb->local_nls);
+		cFYI(1,("\nSetEOF (truncate) rc = %d",rc));
+		if(rc == -ETXTBSY) {        
+			cifsInode = CIFS_I(inode);
+			if(!list_empty(&(cifsInode->openFileList))) {            
+				open_file = list_entry(cifsInode->openFileList.next,
+					struct cifsFileInfo, flist);           
+            /* We could check if file is open for writing first */
+				 rc = CIFSSMBSetFileSize(xid, pTcon, inode->i_size,
+					open_file->netfid,open_file->pid,FALSE);
+			} else {
+				  cFYI(1,("\nNo open files to get file handle from"));
+			}
+		}
 		if (!rc)
-			rc = CIFSSMBSetAllocationSize(xid, pTcon,
-						      full_path,
-						      inode->i_size,
-						      cifs_sb->local_nls);
-
+			CIFSSMBSetEOF(xid,pTcon,full_path,inode->i_size,TRUE,cifs_sb->local_nls);
+           /* allocation size setting seems optional so ignore return code */
 	}
 	if (full_path)
 		kfree(full_path);
@@ -538,13 +578,15 @@ void cifs_truncate_file(struct inode *inode)
 	return;
 }
 
-int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
+int
+cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 {
 	int xid;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
 	int rc = -EACCES;
+	struct cifsFileInfo *open_file = NULL;
 	FILE_BASIC_INFO time_buf;
 	int set_time = FALSE;
 	__u64 mode = 0xFFFFFFFFFFFFFFFF;
@@ -557,22 +599,39 @@ int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 	cFYI(1,
 	     ("\nIn cifs_setattr, name = %s attrs->iavalid 0x%x\n",
 	      direntry->d_name.name, attrs->ia_valid));
-	cifs_sb = (struct cifs_sb_info *) (&(direntry->d_inode->i_sb->u));
+	cifs_sb = CIFS_SB(direntry->d_inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
 	full_path = build_path_from_dentry(direntry);
+	cifsInode = CIFS_I(direntry->d_inode);
 
-
-	/* BB refresh inode from server now ? BB */
+	/* BB check if we need to refresh inode from server now ? BB */
 
 	cFYI(1, ("\nChanging attributes 0x%x", attrs->ia_valid));
 
 	if (attrs->ia_valid & ATTR_SIZE) {
-		rc = CIFSSMBSetEOF(xid, pTcon, full_path, attrs->ia_size,
+		rc = CIFSSMBSetEOF(xid, pTcon, full_path, attrs->ia_size,FALSE,
 				   cifs_sb->local_nls);
-		CIFSSMBSetAllocationSize(xid, pTcon, full_path, attrs->ia_size, cifs_sb->local_nls);	/* dont really care whether works */
+		cFYI(1,("\nSetEOF (setattrs) rc = %d",rc));
+
+		if(rc == -ETXTBSY) {
+			if(!list_empty(&(cifsInode->openFileList))) {            
+				open_file = list_entry(cifsInode->openFileList.next, 
+					   struct cifsFileInfo, flist);           
+    /* We could check if file is open for writing first */
+				rc = CIFSSMBSetFileSize(xid, pTcon, attrs->ia_size,
+					   open_file->netfid,open_file->pid,FALSE);           
+			} else {
+				cFYI(1,("\nNo open files to get file handle from"));
+			}
+		}
+        /*  Set Allocation Size of file - might not even need to call the
+            following but might as well and it does not hurt if it fails */
+		CIFSSMBSetEOF(xid, pTcon, full_path, attrs->ia_size, TRUE, cifs_sb->local_nls);
 		if (rc == 0)
 			vmtruncate(direntry->d_inode, attrs->ia_size);
+		/* BB add special case to handle sharing violation (due to Samba bug)
+		   by calling SetFileInfo to set the sizes */
 	}
 	if (attrs->ia_valid & ATTR_UID) {
 		cFYI(1, ("\nCIFS - UID changed to %d", attrs->ia_uid));
@@ -595,7 +654,7 @@ int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 		CIFSSMBUnixSetPerms(xid, pTcon, full_path, mode, uid, gid,
 				    cifs_sb->local_nls);
 	else {			/* BB to be implemented - via Windows security descriptors */
-		/* CIFSSMBWinSetPerms(xid,pTcon, full_path,mode,uid,gid,cifs_sb->local_nls); */
+		/* CIFSSMBWinSetPerms(xid,pTcon,full_path,mode,uid,gid,cifs_sb->local_nls);*/
 	}
 
 	if (attrs->ia_valid & ATTR_ATIME) {
@@ -614,20 +673,22 @@ int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 
 	if (attrs->ia_valid & ATTR_CTIME) {
 		set_time = TRUE;
-		cFYI(1, ("\nCIFS - CTIME changed "));	/* BB probably do not need to set this */
+		cFYI(1, ("\nCIFS - CTIME changed ")); /* BB probably do not need */
 		time_buf.ChangeTime =
 		    cpu_to_le64(cifs_UnixTimeToNT(attrs->ia_ctime));
 	} else
 		time_buf.ChangeTime = 0;
 
-	if (set_time) {		/* BB handle errors better if one attribute not set (such as size) but time setting works */
+	if (set_time) {		
+        /* BB handle errors better if one attribute not set 
+            (such as size) but time setting works */
 		time_buf.CreationTime = 0;	/* do not change */
-		time_buf.Attributes = 0;	/* BB is this ignored by server?  or do I have to query and reset anyway BB */
+		time_buf.Attributes = 0;	/* BB is this ignored by server?  
+                        or do I have to query and reset anyway BB */
 		rc = CIFSSMBSetTimes(xid, pTcon, full_path, &time_buf,
 				     cifs_sb->local_nls);
 	}
-	cifsInode = (struct cifsInodeInfo *) &(direntry->d_inode->u);
-	cifsInode->time = 0;	/* will force revalidate to go get attributes when needed */
+	cifsInode->time = 0; /* force revalidate to get attributes when needed */
 
 	if (full_path)
 		kfree(full_path);
@@ -635,10 +696,11 @@ int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 	return rc;
 }
 
-void cifs_delete_inode(struct inode *inode)
+void
+cifs_delete_inode(struct inode *inode)
 {
 	/* Note: called without the big kernel filelock - remember spinlocks! */
 	cFYI(1, ("In cifs_delete_inode, inode = 0x%p\n", inode));
-	/* may have to add back in when safe distributed caching of directories via e.g. FindNotify added */
-
+	/* may have to add back in when safe distributed caching of
+             directories via e.g. FindNotify added */
 }
