@@ -31,9 +31,10 @@
 #include "cifsproto.h"
 #include "cifs_debug.h"
 
+extern kmem_cache_t *cifs_mid_cachep;
 
-struct mid_q_entry *AllocMidQEntry(struct smb_hdr *smb_buffer,
-				   struct cifsSesInfo *ses)
+struct mid_q_entry *
+AllocMidQEntry(struct smb_hdr *smb_buffer, struct cifsSesInfo *ses)
 {
 	struct mid_q_entry *temp;
 
@@ -42,11 +43,13 @@ struct mid_q_entry *AllocMidQEntry(struct smb_hdr *smb_buffer,
 		cERROR(1, ("\nNull session passed in to AllocMidQEntry "));
 		return NULL;
 	}
-	temp = kmalloc(sizeof(struct mid_q_entry), GFP_KERNEL);
+	temp = kmalloc(sizeof (struct mid_q_entry), GFP_KERNEL);
+	temp = (struct mid_q_entry *) kmem_cache_alloc(cifs_mid_cachep,
+						       SLAB_KERNEL);
 	if (temp == NULL)
 		return temp;
 	else {
-		memset(temp, 0, sizeof(struct mid_q_entry));
+		memset(temp, 0, sizeof (struct mid_q_entry));
 		temp->mid = smb_buffer->Mid;	/* always LE */
 		temp->pid = current->pid;
 		temp->command = smb_buffer->Command;
@@ -56,46 +59,57 @@ struct mid_q_entry *AllocMidQEntry(struct smb_hdr *smb_buffer,
 #else
 		do_gettimeofday(&temp->when_sent);
 #endif
+
 		temp->ses = ses;
 		temp->tsk = current;
 	}
 	if (ses->status == CifsGood) {
+		write_lock(&GlobalMid_Lock);
 		list_add_tail(&temp->qhead, &ses->server->pending_mid_q);
 		atomic_inc(&midCount);
+		temp->midState = MID_REQUEST_ALLOCATED;
+		write_unlock(&GlobalMid_Lock);
 	} else {		/* BB add reconnect code here BB */
 
 		cERROR(1,
 		       ("\nNeed to reconnect after session died to server\n"));
 		if (temp)
-			kfree(temp);
+			kmem_cache_free(cifs_mid_cachep, temp);
 		return NULL;
 	}
 	return temp;
 }
 
-void DeleteMidQEntry(struct mid_q_entry *midEntry)
+void
+DeleteMidQEntry(struct mid_q_entry *midEntry)
 {
 	/* BB add spinlock to protect midq for each session BB */
-	buf_release(midEntry->resp_buf);
+	write_lock(&GlobalMid_Lock);
+	midEntry->midState = MID_FREE;
 	list_del(&midEntry->qhead);
-	kfree(midEntry);
 	atomic_dec(&midCount);
+	write_unlock(&GlobalMid_Lock);
+	buf_release(midEntry->resp_buf);
+	kmem_cache_free(cifs_mid_cachep, midEntry);
 }
 
-int smb_send(struct socket *ssocket, struct smb_hdr *smb_buffer,
-	     unsigned int smb_buf_length, struct sockaddr *sin)
+int
+smb_send(struct socket *ssocket, struct smb_hdr *smb_buffer,
+	 unsigned int smb_buf_length, struct sockaddr *sin)
 {
 	int rc = 0;
 	struct msghdr smb_msg;
 	struct iovec iov;
 	mm_segment_t temp_fs;
 
+	if(ssocket == NULL)
+		return -ENOTSOCK; /* BB eventually add reconnect code here */
 /*  ssocket->sk->allocation = GFP_BUFFER; *//* BB is this spurious? */
 	iov.iov_base = smb_buffer;
 	iov.iov_len = smb_buf_length + 4;
 
 	smb_msg.msg_name = sin;
-	smb_msg.msg_namelen = sizeof(struct sockaddr);
+	smb_msg.msg_namelen = sizeof (struct sockaddr);
 	smb_msg.msg_iov = &iov;
 	smb_msg.msg_iovlen = 1;
 	smb_msg.msg_control = NULL;
@@ -107,8 +121,11 @@ int smb_send(struct socket *ssocket, struct smb_hdr *smb_buffer,
 	   len is converted to bigendian in smb_send */
 	if (smb_buf_length > 12)
 		smb_buffer->Flags2 = cpu_to_le16(smb_buffer->Flags2);
-	smb_buffer->smb_buf_length =
-	    cpu_to_be32(smb_buffer->smb_buf_length);
+
+    /* if(smb_buffer->Flags2 & SMBFLG2_SECURITY_SIGNATURE)
+        sign_smb(smb_buffer); */ /* BB enable when signing tested more */
+
+	smb_buffer->smb_buf_length = cpu_to_be32(smb_buffer->smb_buf_length);
 	cFYI(1, ("\nSending smb of length %d ", smb_buf_length));
 	dump_smb(smb_buffer, smb_buf_length + 4);
 
@@ -120,17 +137,17 @@ int smb_send(struct socket *ssocket, struct smb_hdr *smb_buffer,
 
 	if (rc < 0) {
 		cERROR(1,
-		       ("\nError %d sending data on socket to server.\n",
-			rc));
+		       ("\nError %d sending data on socket to server.\n", rc));
 	} else
 		rc = 0;
 
 	return rc;
 }
 
-int SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
-		struct smb_hdr *in_buf, struct smb_hdr *out_buf,
-		int *pbytes_returned, const int long_op)
+int
+SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
+	    struct smb_hdr *in_buf, struct smb_hdr *out_buf,
+	    int *pbytes_returned, const int long_op)
 {
 	int rc = 0;
 	int receive_len;
@@ -147,23 +164,28 @@ int SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 		DeleteMidQEntry(midQ);
 		return -EIO;
 	}
+	midQ->midState = MID_REQUEST_SUBMITTED;
 	rc = smb_send(ses->server->ssocket, in_buf, in_buf->smb_buf_length,
 		      (struct sockaddr *) &(ses->server->sockAddr));
 
-	cFYI(1, ("\ncifs smb_send rc %d", rc));	/* BB remove */
-	/* BB add code to wait for response and copy to out_buf */
 	if (long_op > 1)	/* writes past end of file can take a looooooong time */
 		timeout = 300 * HZ;
 	else if (long_op == 1)
 		timeout = 60 * HZ;
 	else
 		timeout = 15 * HZ;
-	/* wait for 15 seconds or until woken up due to response arriving or due to last connection to this server being unmounted */
-	timeout =
-	    interruptible_sleep_on_timeout(&ses->server->response_q,
-					   timeout);
-	cFYI(1, (" with timeout %ld and Out_buf: %p  midQ->resp_buf: %p ",
-		 timeout, out_buf, midQ->resp_buf));
+	/* wait for 15 seconds or until woken up due to response arriving or due to 
+	   last connection to this server being unmounted */
+
+	timeout = interruptible_sleep_on_timeout(&ses->server->response_q,timeout);
+	/* Until 2.5 can not use wait_event with timeout so stuck with sleep_on */
+	/*timeout = wait_event_interruptible_timeout(ses->server->response_q,
+					midQ->
+					midState & MID_RESPONSE_RECEIVED,
+					timeout); */
+	cFYI(1,
+	     (" with timeout %ld and Out_buf: %p  midQ->resp_buf: %p ", timeout,
+	      out_buf, midQ->resp_buf));
 	if (signal_pending(current)) {
 		cERROR(1, (KERN_ERR "\nCIFS: caught signal"));
 		DeleteMidQEntry(midQ);
@@ -189,7 +211,8 @@ int SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 		rc = -EIO;
 	} else {		/* rcvd frame is ok */
 
-		if (midQ->resp_buf && out_buf) {
+		if (midQ->resp_buf && out_buf
+		    && (midQ->midState == MID_RESPONSE_RECEIVED)) {
 			memcpy(out_buf, midQ->resp_buf,
 			       receive_len +
 			       4 /* include 4 byte RFC1001 header */ );
@@ -200,15 +223,13 @@ int SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 			out_buf->smb_buf_length =
 			    be32_to_cpu(out_buf->smb_buf_length);
 			if (out_buf->smb_buf_length > 12)
-				out_buf->Flags2 =
-				    le16_to_cpu(out_buf->Flags2);
+				out_buf->Flags2 = le16_to_cpu(out_buf->Flags2);
 			if (out_buf->smb_buf_length > 28)
 				out_buf->Pid = le16_to_cpu(out_buf->Pid);
 			if (out_buf->smb_buf_length > 28)
 				out_buf->PidHigh =
 				    le16_to_cpu(out_buf->PidHigh);
 
-
 			*pbytes_returned = out_buf->smb_buf_length;
 
 			/* BB special case reconnect tid and reconnect uid here? */
@@ -216,7 +237,7 @@ int SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 
 			/* convert ByteCount if necessary */
 			if (receive_len >=
-			    sizeof(struct smb_hdr) -
+			    sizeof (struct smb_hdr) -
 			    4 /* do not count RFC1001 header */  +
 			    (2 * out_buf->WordCount) + 2 /* bcc */ )
 				BCC(out_buf) = le16_to_cpu(BCC(out_buf));
@@ -224,6 +245,8 @@ int SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 			rc = -EIO;
 	}
 
-	DeleteMidQEntry(midQ);	/* BB what if process is killed ? - BB add background daemon to clean up Mid entries from killed processes BB test killing process with active mid */
+	DeleteMidQEntry(midQ);	/* BB what if process is killed?
+			 - BB add background daemon to clean up Mid entries from
+			 killed processes & test killing process with active mid */
 	return rc;
 }
